

class MutableIndex:

    def __init__(self):
        pass

    def value(self, filename: str):
        """
        Traverse over AST tree and finds loops with mutable index
        :return:
        List of line number of loops with mutable index
        @todo #147:30min Implement MutableIndex pattern
         We need to implement the code on mutable index pattern. It must return the number
         of the lines where we have an index of a for loop being mutated. After implementing,
         the method, enable all tests in test_mutable_index.py
        """
        return []
